/**
 * File: conf_usb.h
 * Creator: Jonathan Klapel
 * Date: 08 June 2014
 * Description: Configuration file for USB
 */ 

#ifndef _CONF_USB_H_
#define _CONF_USB_H_

#include "compiler.h"

// Device definition (mandatory)
#define  USB_DEVICE_VENDOR_ID             USB_VID_ATMEL
#define  USB_DEVICE_PRODUCT_ID            USB_PID_ATMEL_ASF_HIDGENERIC
#define  USB_DEVICE_MAJOR_VERSION         1
#define  USB_DEVICE_MINOR_VERSION         0
#define  USB_DEVICE_POWER                 100 // Consumption on Vbus line (mA)
#define  USB_DEVICE_ATTR                  \
	(USB_CONFIG_ATTR_REMOTE_WAKEUP|USB_CONFIG_ATTR_BUS_POWERED)
//	(USB_CONFIG_ATTR_REMOTE_WAKEUP|USB_CONFIG_ATTR_SELF_POWERED)
//	(USB_CONFIG_ATTR_REMOTE_WAKEUP|USB_CONFIG_ATTR_BUS_POWERED)
//	(USB_CONFIG_ATTR_SELF_POWERED)
//	(USB_CONFIG_ATTR_BUS_POWERED)

// USB Device string definitions (Optional)
#define  USB_DEVICE_MANUFACTURE_NAME      "ATMEL ASF"
#define  USB_DEVICE_PRODUCT_NAME          "HID Generic"
// #define  USB_DEVICE_SERIAL_NAME           "12...EF"

// To define a Low speed device
//#define  USB_DEVICE_LOW_SPEED

//! To authorize the High speed
#if (UC3A3||UC3A4)
//#define  USB_DEVICE_HS_SUPPORT
#elif (SAM3XA||SAM3U)
//#define  USB_DEVICE_HS_SUPPORT
#endif

// USB Device Callbacks definitions (Optional)
#define  UDC_VBUS_EVENT(b_vbus_high)
#define  UDC_SOF_EVENT()                  ui_process()
#define  UDC_SUSPEND_EVENT()              ui_powerdown()
#define  UDC_RESUME_EVENT()               ui_wakeup()
// Mandatory when USB_DEVICE_ATTR authorizes remote wakeup feature
#define  UDC_REMOTEWAKEUP_ENABLE()        ui_wakeup_enable()
#define  UDC_REMOTEWAKEUP_DISABLE()       ui_wakeup_disable()
// When a extra string descriptor must be supported
// other than manufacturer, product and serial string
// #define  UDC_GET_EXTRA_STRING()



// Interface callback definition
#define  UDI_HID_GENERIC_ENABLE_EXT()        main_generic_enable()
#define  UDI_HID_GENERIC_DISABLE_EXT()       main_generic_disable()
#define  UDI_HID_GENERIC_REPORT_OUT(ptr)     ui_report_out(ptr)
#define  UDI_HID_GENERIC_SET_FEATURE(report) main_hid_set_feature(report)

// Sizes of I/O reports
#define  UDI_HID_REPORT_IN_SIZE             8
#define  UDI_HID_REPORT_OUT_SIZE            8
#define  UDI_HID_REPORT_FEATURE_SIZE        4

// Sizes of I/O endpoints
#define  UDI_HID_GENERIC_EP_SIZE            8

// The includes of classes and other headers must be done at the end of this file to avoid compile error
#include "udi_hid_generic_conf.h"
#include "main.h"
#include "ui.h"

#endif // _CONF_USB_H_
