/**
 * File: Zigbit.h
 * Creator: Jonathan Klapel
 * Date: 08 June 2014
 * Description: Performs operations with Zigbit modules
 */ 


#ifndef ZIGBIT_H_
#define ZIGBIT_H_

#include <string.h>
#include "conf_board.h"
#include "avr2025_mac.h"
#include "led.h"
#include "delay.h"
#include "common_sw_timer.h"
#include "sleepmgr.h"
#include <asf.h>

/* === TYPES =============================================================== */

typedef struct associated_device_tag {
	uint16_t short_addr;
	uint64_t ieee_addr;
}

associated_device_t;

typedef enum node_status_tag {
	COLD_START_RESET,
	PC_RUNNING,
	IDLE,
	TRANSMITTING
} SHORTENUM node_status_t;

typedef enum msg_type_t {
	UNKNOWN,
	SYNC_PULSE,
	ACK_PULSE,
	TIMESTAMP_PULSE,
	CONFIRM_SYNC,
	START_RECORDING,
	RECORDING_STATUS,
	ALTER_WAV_FILE
} SHORTENUM msg_type;

/* === MACROS ============================================================== */

#ifdef PAN_ID
#define DEFAULT_PAN_ID                  (PAN_ID)
#else
#define DEFAULT_PAN_ID                  CCPU_ENDIAN_TO_LE16(0xBABE)
#endif
/** Defines the short address of the coordinator. */
#define COORD_SHORT_ADDR                (0x0000)
/** Defines the maximum number of devices the coordinator will handle. */
#define MAX_NUMBER_OF_DEVICES           (1)
/** This is the time period in micro seconds for data transmissions. */
#define DATA_TX_PERIOD                  (2000000)

#define CHANNEL_OFFSET                  (0)

#define SCAN_CHANNEL                    (1ul << current_channel)

/** Defines the scan duration time. */
#define SCAN_DURATION                   (4)
/** Defines the maximum number of scans before starting own network. */
#define MAX_NUMBER_OF_SCANS             (3)

#if (LED_COUNT >= 3)
#define LED_START                       (LED0)
#define LED_NWK_SETUP                   (LED1)
#define LED_DATA                        (LED2)
#elif (LED_COUNT == 2)
#define LED_START                       (LED0)
#define LED_NWK_SETUP                   (LED0)
#define LED_DATA                        (LED1)
#else
#define LED_START                       (LED0)
#define LED_NWK_SETUP                   (LED0)
#define LED_DATA                        (LED0)
#endif

#define INTER_FRAME_DURATION_US         (200000)

#define TASK_2048_STACK_SIZE				(2048/sizeof(portSTACK_TYPE))
#define TASK_MONITOR_STACK_PRIORITY			(tskIDLE_PRIORITY)
#define TASK_1024_STACK_SIZE                (1024/sizeof(portSTACK_TYPE))
#define TASK_STACK_PRIORITY_0				0
#define TASK_STACK_PRIORITY_1				1
#define TASK_STACK_PRIORITY_2				2

/* === GLOBALS ============================================================= */

/** This array stores all device related information. */
associated_device_t device_list[MAX_NUMBER_OF_DEVICES];
/** Current status of the node. */
node_status_t node_status;

uint8_t current_channel;
uint8_t current_channel_page;
uint32_t channels_supported;

uint8_t TIMER_LED_OFF;

associated_device_t coord_addr;

/* === PROTOTYPES ========================================================== */

static void network_scan_indication_cb(void *parameter);
static void data_exchange_led_off_cb(void *parameter);
static bool assign_new_short_addr(uint64_t addr64, uint16_t *addr16);
uint8_t send_message(msg_type msgtype, uint8_t length, uint8_t * data);

/* === IMPLEMENTATION ====================================================== */
/* These must be implemented in the main.c */
void wireless_ready(void);
void data_received(uint8_t msduLength, uint8_t *msdu, uint32_t Timestamp);
void data_transmitted(uint8_t msduHandle, uint8_t status, uint32_t Timestamp);


#endif /* ZIGBIT_H_ */