function [ output_args ] = ir_response(lsp_file,mic_file,window_size,plot_title,smoothing_constant)
%ir_response Computes and displays the impulse response of a recording
%   Computes and displays the impulse response of a recording
%
% Parameters:
%  lsp_file: Loudspeaker Output File (WAV)
%  mic_file: Microphone Recording File (WAV)
%  window_size: IR window size to use
%  plot_title: Title of the plot
%  smoothing_constant: Constant to use when smoothing plot

%Read input files
[sig,fs] = wavread(lsp_file);
[micrec, fs_rec] = wavread(mic_file);

%Check that Sampling Frequencies are equal
if fs ~= fs_rec
    error('Sample Frequency Mismatch');
end

%Make sure we are only using mono files
sig = sig(:,1);
micrec = micrec(:,1);

%Add default values
if nargin < 5
    smoothing_constant = 1;
end
if nargin < 4
    plot_title = mic_file;
end
if nargin < 3
    window_size = size(micrec,1);
end

%Begin analysis
nfft_analyze = 2^20;
TF_system = fft(micrec,nfft_analyze)./fft(sig,nfft_analyze);
ir_system = real(ifft(TF_system));
ivdirectsoundwindow = [1:window_size];
ir_useful = ir_system(ivdirectsoundwindow);

%write the IR out
wavwrite(ir_useful,fs,plot_title);

%Plot Time Domain IR
figure(1)
plot(ir_useful)
title(strcat('Time Domain IR - ',plot_title));
xlabel('Sample');
ylabel('Response');

%Plot Frequency Domain IR
nfft_plot = 16384;
TF_useful = fft(ir_useful,nfft_plot);
fvec = fs/nfft_plot*[0:nfft_plot/2-1];
figure(2)
semilogx(fvec,smooth(20*log10(abs(TF_useful(1:nfft_plot/2))),1))
title(strcat('Frequency Domain IR - ',plot_title));
xlabel('Frequency');
ylabel('Response (db scale)');
axis([0 100000 -100 20]);

figure(3)
semilogx(fvec,smooth(20*log10(abs(TF_useful(1:nfft_plot/2))),smoothing_constant))
title(strcat('Frequency Domain IR - ',plot_title,'(smoothed)'));
xlabel('Frequency');
ylabel('Response (dB scale)');
axis([0 100000 -100 20]);

%Save images
%saveas(1,strcat('Time Domain IR - ',plot_title,'.png'));
%saveas(2,strcat('Frequency Domain IR - ',plot_title,'.png'));
%saveas(3,strcat('Frequency Domain IR - ',plot_title,'(smoothed).png'));

end

